dojo.provide("com.ibm.pb.customize.enabler.AddContentController");

dojo.require("com.ibm.pb.data.InstalledPortletStore");
dojo.require("com.ibm.pb.data.WcmStore");
dojo.require("com.ibm.data.JsonStore");
dojo.require("com.ibm.portal.fx.Transitions");
dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.require("dojo.DeferredList");
dojo.require("com.ibm.pb.layout.LayoutParser");
dojo.requireLocalization("com.ibm.bundles", "Shelf");

com.ibm.pb.customize.enabler.AddContentController = {
	dirty: false,			// true if there are changes pending
	started: false,			// true if the widget is started up
	refresh: true,			// true if a page refresh is required to show saved changes
	navModel: null,			// reference to enabler navigation model
	imagePath: "",			// path to CSS images
	pendingItems: {},		// map of controls on the page
	placeholders: [],		// array of SSA portlet placeholders
	ibmMainContainer: "ibmMainContainer",
    layoutParser: com.ibm.pb.layout.LayoutParser,

	startup: function(){
		this.logger.entering("startup");
		this.started = true;	// widget is atarted
		this.eventService = com.ibm.mashups.iwidget.services.ServiceManager.getService("eventService");
		this.setupUI();
		this.logger.exiting("startup");
	},
		
	makeDirty: function(/*bool*/change){
		// summary: changes the dirty attribute
		// change: value to assign to the dirty attribute
		this.logger.entering("makeDirty",[change]);
		this.dirty = change;
		this.logger.exiting("makeDirty",[change]);
	},
		
	processChange: function(/*Object*/args){
		// summary: used as an event handler for when a user clicks on an item
		//			hub: the datastore to which the object belongs
		//			item: the item in the datastore being processed
		//			prefsMapFcn (optional): a function or the name of a function in dot notation which retrieves a map of the preferences for the piece of content
		//			takeoverFcn (optional 2): a function or the name of a function in dot notation which will be run instead of the body of this submit function
		//			takeoverObject (optional 2): an object of parameters to be passed to takeoverFcn, must have both takeoverFcn and takeoverObject
		//			cID (optional 3): id of the container in which to insert the control
		//			sID (optional 3): goes with cID, it is the sibling id before which to insert the control
		if(!this.dirty) { this.makeDirty(true); }
		if(args.takeoverFcn) {
			// if there is a takeover function, pass control on to it
			args.takeoverObject = dojo.mixin(args.takeoverObject, {controller:this});
			if(dojo.isString(args.takeoverFcn)) { return dojo.getObject(args.takeoverFcn)(args.takeoverObject); }
			else { return args.takeoverFcn(args.takeoverObject); }
		} else { return this.apply(args); }
	},
	
	_canProcess: function(id) {
		// return true if the item with the given id is allowed
		if(!ibmCfg.portalConfig.portalContext || !id) return true;  // Mashups always returns true
		var node = this.widgetModel.find(new com.ibm.pb.customize.Identifiable(id)).start();
		var prefs = node.getPreferences();
		if(prefs.getValue("WCM_CONTENT_TYPE")) {
			// this is a WCM portlet, it can only be processed if the page has a content mapping
			if(this._pageHasContentMapping()) return true;
			else {
				var bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
				var statusMessage = new com.ibm.widgets.StatusMessage("error", bundle["no_web_content_mapping"] || "Cannot add this item to the page", bundle["no_web_content_mapping_desc"] || "Please add a Web Content mapping to the page before adding this item.");
				dojo.publish("/message/status",[{message: statusMessage, uid: 'ibmStatusBox'}]);
				return false;
			}
		} else return true;
	},
		
	apply: function(/*Object*/args){
		// summary: applies a change associated with a selected item to the page
		// args: same as processChange
		this.logger.entering("apply",[args]);

		// 20913: make sure the dialog object is available
		if (this.dialog) {
		    this.dialog.hide();
		}
		
		// check if this application is allowed
		var id = args.hub.mapItem(args.item).id;
		if(!this._canProcess(id)) return;
		
		// get layout model
		var navId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		var layoutModel = this.navModel.getLayoutModel(navId);
		
		// get container into which the widget/portlet will go
		var container = null,
			containerDef = null,
			containerId = "";
		if(args.cID) { 
			//get container id from dnd
			containerId = args.cID; 
			containerDef = layoutModel.find(containerId);
			if(containerDef) {
				containerDef.setFinishedCallback(dojo.hitch(this, "_apply"), {layoutModel:layoutModel, args:args});
				containerDef.start(false);
			} else { this.logger.severe("apply","Cannot find dnd container in the layout model",[args]); }
		} else {
			// get the container from the layout model
			var mainContainer = dojo.query("[name~="+this.ibmMainContainer+"]");
			if(mainContainer.length > 0){
				// get the container which has the ibmMainContainer name
				containerId = this.layoutParser.extractID(mainContainer[0]);
				containerDef = layoutModel.find(containerId);
				if(containerDef) {
					containerDef.setFinishedCallback(dojo.hitch(this, "_apply"), {layoutModel:layoutModel, args:args});
					containerDef.start(false);
				} else { this.logger.severe("apply","Cannot find main container in the layout model",[args]); }
			} else {
				// get first container on the page
				var rootDef = layoutModel.getRoot();
				if(rootDef) { var root = rootDef.start(); }
				else { this.logger.severe("apply","Cannot find root layout container in the model",[args]); return; }
				var iter = layoutModel.getChildren(root);
				while(iter.hasNext()){
					container = iter.next();
					if(dojo.byId(container.getID())) { break; }
				}
				if(container) { this._apply(container, "200", {layoutModel:layoutModel, args:args});  }
				else { this.logger.severe("apply","Cannot find any containers in the layout model",[args]); }
			}
		}
		this.logger.exiting("apply",[args]);
	},
	
	_apply: function(container, status, x){
		// summary: inserts a control onto the page
		// container: layout container into which to put the new control
		// status: HTTP status returned from request to get container
		// x: {layoutModel, args from apply()}
		this.logger.entering("_apply",[container, status, x]);
		if(status != 200) {
			this.logger.severe("_apply","Could not find container in the layout model.  Cannot Add Content",[container, status, x]);
			return;
		}
		
		if(container instanceof com.ibm.mashups.enabler.layout.LayoutContainer){
			// get the next sibling
			var nextSibling = null;
			if(x.args.cID && !x.args.sID){
				nextSibling = null;
			} else if(x.args.sID){
				// get the next sibling from dnd
				var controlId = x.args.sID;
				if(controlId) { nextSiblingDef = x.layoutModel.find(controlId); }
				if(nextSiblingDef) { nextSibling = nextSiblingDef.start(); }
				if(!nextSibling) { this.logger.warning("_apply","Cannot find dnd sibling control in the layout model.  Setting it to null.",[container, status, x]); }
			} else {
				// get the first control in the container
				var iter = x.layoutModel.getChildren(container);
				if(iter.hasNext()) { nextSibling = iter.next(); }
			}
			if(nextSibling && !(nextSibling instanceof com.ibm.mashups.enabler.layout.LayoutControl)) { 
				// make sure nextSiblig is a Layout Control object
				this.logger.warning("_apply","Next sibling control is not an instance of com.ibm.mashups.enabler.layout.LayoutControl.  Setting it to null.",[container, status, x]);
				nextSibling = null;
			}
			
			// create the new control instance
			var constants = com.ibm.mashups.enabler.layout.Constants;
			var context = {};
			context[constants.CONTEXT_TYPE] = constants.LAYOUT_CONTROL;
            if (x.args.definitionURL) {
                context[constants.CONTEXT_WIDGET_DEFINITION_URL] = x.args.definitionURL;
            } else {
                context[constants.CONTEXT_WIDGET_DEFINITION_ID] = new com.ibm.pb.customize.Identifiable(x.args.id);
            }
            context[constants.DISPLAY_TITLE] = x.args.label;
			var control = x.layoutModel.create(context);
			if(!ibmCfg.portalConfig.portalContext){
				// temp fix for 18222 - Portal only
				control.setSkin(this._getDefaultSkin());
			}
			
			if (x.args.widgetType === "mm_OSGadget") {
				// gadgets do not have translated titles in their definition,
				// so the titles are forced in from the catalog for gadgets
				var titles = {};
				if (x.args.titles) {
					for (var i = 0; i < x.args.titles.childNodes.length; i++) {
						if (x.args.titles.childNodes[i].nodeType === 1) {
							titles[x.args.titles.childNodes[i].attributes[0].textContent] = x.args.titles.childNodes[i].textContent;
						}
					}
				}
				
				for (var loc in titles) {
					if (titles[loc] !== "{$data}") {
						control.setTitle(titles[loc],loc);
					}
				}
			} else {
				var defaultTitle=this._getWidgetDefinitionTitle(x.args.definitionURL);
				if(defaultTitle !=x.args.label){
					control.setTitle(x.args.label,dojo.locale);
				}
			}
			
			this._addLayoutControlMetaData(control, x.args);
			x.layoutModel.insert(control, container, nextSibling);
			this.pendingItems[x.args.label] = control;
			
			// get and assign prefs
			if(x.args.prefsMapFcn) { this._addPreferences(control, x.args.prefsMapFcn, x.args); }
			
			//set sandbox mark
			this._setSandboxMark(control, !x.args.isTrusted);
			
			// refresh the layout to show the new widget
			this._addPreviewToPage(control, container, nextSibling, x.args.label);
			
		} else { this.logger.severe("_apply","Container is not an instance of com.ibm.mashups.enabler.layout.LayoutContainer",[container, status, x]); }
		com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(true);
		this.logger.exiting("_apply",[container, status, x]);
	},
	_getWidgetDefinitionTitle:function(definitionURL){
		if(!definitionURL) { return null; }  // Portlets do not have definition URLs, so this functions breaks in Portal
		var globalModel = com.ibm.mashups.iwidget.model.Factory.getGlobalWidgetModel();
		var widgetDef = globalModel.getWidgetDefinitionByURL(definitionURL).start(true);
		try {
			return widgetDef.getItemSet("idescriptor").items.title.values[dojo.locale];
		}catch(e){
			return null;
		}
	},
	
	_addPreviewToPage: function(control, container, nextSibling, title) {
		// summary: adds the markup to the page for live preview of the new widget
		// control: the control instance added to the page
		// container: the container instance to which the control was added
		// nextSibling: if present, the control was inserted before this other control
		// title: title of the control
		var getPreviewHTML = control.getRuntimeContent().getRuntimeMarkup();
		var me = this;
		getPreviewHTML.setFinishedCallback(function(markup){
			// put the runtime markup on the page
			if(ibmCfg.portalConfig.portalContext){
				// temp fix for 18222 and Portal blog&wiki UAT breakage - Portal only!
				var i = markup.match(/id-([^"]+)/);
				i = i && i[1];
				markup = '<div class="component-control id-'+i+'"><div class="lotusWidget2"><h2 class="dojoDndHandle"><span>' + title + '</span></h2><div class="lotusWidgetBody"><div class="iw-iWidget" id="'+i+'"><a class="iw-Definition" href="/mccenabler/js/com/ibm/mm/enabler/layout/widget/NonPreviewableContentWidget.xml"></a></div></div></div></div>';
			}
			var controlId = control.getID(), containerId = container.getID();
			if (nextSibling) {
				var sibling = me.layoutParser.findById(nextSibling.getID());
				var newWidget = dojo.place(markup, sibling, "before");
			} else {
				var parent = me.layoutParser.findById(containerId);
				var newWidget = dojo.place(markup, parent, "last");
			}
			// add dnd attributes
			// if you run into problems here with newWidget not a correct DOM Node, this is most likely
			// due to LayoutContainerFragment.html and/or LayoutControlFragment.html containing leading or trailing whitespaces.
			// If so, remove them before building, because dojo.place behaves strange otherwise and returns a DOM fragment instead of a real DOM Node.
			dojo.addClass(newWidget, "dojoDndItem"); 
			dojo.attr(newWidget, "dndType", "lm");
			// extend the sandboxed widget height to 100%
			dojo.forEach(
				dojo.query(".iw-iWidget[sandbox='true']",newWidget),
				function(sandboxed){
					dojo.addClass(sandboxed,"extendediwiWidget");
				}
			);				
			// publish events to parse, add, and wire the widget
			dojo.publish("/com/ibm/mashups/livetext/livetextchanged", [newWidget, true]);
			me.eventService.broadcastEvent("com.ibm.mashups.builder.addNewWidget", controlId); 
			com.ibm.mm.builder.utils.wireUtil.triggerAutowire(controlId);	
			ibmCfg.controllers.dnd.dndSources[containerId].sync();
		});
		getPreviewHTML.start(false); 
	},
	
	_getDefaultSkin: function(){
		// summary: returns the default skin for the theme assigned to the current page
		var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		var page = this.navModel.find(pageId).start();
		var themeId = page.getTheme();
		var theme = com.ibm.mashups.enabler.model.Factory.getThemeModel().find(themeId).start();
		return theme.getDefaultSkin();
	},
	
	_setSandboxMark: function (control, markSandBox) {
		try {
			var windowDef = this.widgetModel.getWidgetWindow(control);
			windowDef.setFinishedCallback(dojo.hitch(this,function(window, status){
				if(status == 200){
					instanceDef = this.widgetModel.getWidgetInstance(window);
					instanceDef.setFinishedCallback(dojo.hitch(this,function(instance, status){
						if(status == 200){
							instance.setSandboxed(markSandBox);
						}
					}));
					instanceDef.start(true);
				}
			}));
			windowDef.start(true);	
		} catch(e) {
			console.log(e);			
		}
	},
	
	_addLayoutControlMetaData: function(control, map) {
		// summary: add meta data to layout control
		// key refers to results from mapItem()
		control.setMetaData("thumbnail", map["thumbnail"]);
		control.setMetaData("onNewWire", map["onNewWire"]);
		control.setMetaData("onAutoWire", map["onAutoWire"]);
		control.setMetaData("icon", map["icon"]);
		control.setMetaData("widgetParam", map["widgetParam"]);
	},
	
	_addPreferences: function(control, prefsMapFcn, map){
		// summary: adds preferences to a control
		// control: layout control to which to add preferences
		// prefsMapFcn: function which gets the preferences
		// map: arguments for the prefsMapFcn
		this.logger.entering("_addPreferences",[control, prefsMapFcn, map]);
		var windowDef = this.widgetModel.getWidgetWindow(control);
		windowDef.setFinishedCallback(dojo.hitch(this,function(window, status){
			if(status == 200){
				instanceDef = this.widgetModel.getWidgetInstance(window);
				instanceDef.setFinishedCallback(dojo.hitch(this,function(instance, status){
					if(status == 200){
						var prefsDef = this.widgetModel.getHierarchicalPreferences(instance);
						prefsDef.setFinishedCallback(dojo.hitch(this,function(preferences, status){
							if(status == 200) {
								var prefsMap = {};
								if(dojo.isString(prefsMapFcn)) { prefsMap = dojo.getObject(prefsMapFcn)(map); }
								else { prefsMap = prefsMapFcn(map); }
								for(var i in prefsMap){ 
									preferences.setValue(i, prefsMap[i]);
								}
							} else { this.logger.warning("_addPreferences","Cannot preferences for widget instance",[control, prefsMapFcn, map]); }
						}));
						prefsDef.start(true);
					} else { this.logger.warning("_addPreferences","Cannot find instance for widget window",[control, prefsMapFcn, map]); }
				}));
				instanceDef.start(true);
			} else { this.logger.warning("_addPreferences","Cannot find widget window based on newly created control",[control, prefsMapFcn, map]); }
		}));
		windowDef.start(true);
		this.logger.exiting("_addPreferences",[control, prefsMapFcn, map]);
	},
		
	undo: function(){
		// summary: reverts any changes made
		this.logger.entering("undo");
		dojo.forEach(this.beforeSubmitDfds, function(dfd){dfd.cancel();});
		this.beforeSubmitDfds = [];
		/*var isSSA = true;
		if(isSSA){
			for(var p in this.placeholders) {
				this.placeholders[p].destroyRecursive();
			}
		} else {
			var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
			this.eventService.broadcastEvent("com.ibm.mashups.builder.layoutRefresh");
		}*/
		this.logger.exiting("undo");
	},
		
	beforeSubmit: function(/*Object*/args){
		// summary: Called before submit and returns a Deferred object which should be used
		// 		to get notified when the controller is ready to be submitted.  Attach a callback
		//		to this Deferred to call the controller's submit function instead of calling
		//		it directly.
		this.logger.entering("beforeSubmit",[args]);
		var dfd = (new dojo.Deferred()).addCallback(dojo.hitch(this, function(){
			// add all the controls to the containers
			// chain up all the functions added with addBeforeSubmit
			this.logger.entering("beforeSubmit dfd callback");
			var dfdl = new dojo.DeferredList(this.beforeSubmitDfds, false, false);
			dojo.forEach(this.beforeSubmitDfds, function(df){
				if(df.fired < 0) {
					df.callback();
				}
			});
			this.beforeSubmitDfds = [];
			this.logger.exiting("beforeSubmit dfd callback");
			return dfdl;	// chain up another deferred after dfd calls back so subsequent callbacks now wait on dfdl
		}));
		// add the error callack to the deferred object
		if(args && args.onError) { dfd.addErrback(args.onError); }
		dfd.callback();
		this.logger.exiting("beforeSubmit",[args]);
		return dfd;
	},
		
	submit: function(/*Object*/args){
		// summary: saves any pending changes
		// args: args may contain an onComplete handler
		this.logger.entering("submit",[args]);
		if(this.dirty) { this.makeDirty(false); }
		this.pendingItems = {};
		if(args.onComplete) { args.onComplete(); }
		this.logger.exiting("submit",[args]);
	},
	
	addBeforeSubmit: function(/*Deferred*/ dfd) {
		// summary: Adds a Deferred object to the list of deferreds that must all be fired before
		//		the Deferred object returned by beforeSubmit is fired. 
		this.logger.entering("addBeforeSubmit",[dfd]);
		if(dfd) { this.beforeSubmitDfds.push(dfd); }
		this.logger.exiting("addBeforeSubmit",[dfd]);
	},

/********************************************************/
/*    			 Takeover functions 				    */
/********************************************************/	

	newFeed: function(/*object*/args){
	    // summary: creates a new Feedspace portlet with user provided feed url
		// args: contains one item, getInfo, which is a function that takes the form data and runs a callback function on the validated user inputs
		var bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
		var form = [{label:bundle["new_feed_title"], id:"new_feed_title", value:"", regex:/\S+/, error:bundle["new_feed_enterTitle"]}, 
					{label:bundle["new_feed_url"], id:"new_feed_url", value:"http://", regex:/http:\/\/\S+/, error:bundle["new_feed_enterUrl"], ltr:true}];
		var submit = dojo.hitch(args.controller, function(inputs){ 
			var label = inputs["new_feed_title"].replace(/<(?:\/?)script>/g,""), url = inputs["new_feed_url"].replace(/<(?:\/?)script>/g,"");
			args.submit(dojo.mixin(args.submitObj,{id:ibmPortalConfig.contentRenderers.feed,label:label})); // add the feed reader to the page
			this.addBeforeSubmit((new dojo.Deferred()).addCallback(dojo.hitch(this, function(){      
				// set a beforeSubmit function to add the feed preferences
				this._addPreferences(this.pendingItems[label], this.getFeedspacePreferences, {url:url,label:label});
			})));
		});
		args.getInfo(form, bundle["new_feed_addFeed"], bundle["add_content_add"], null, submit);
	},
		
	newWCM: function(/*object*/args){
	    // summary: creates a new WCM item like a blog, wiki or solo blog
		// args: an associative array that contains:	getInfo: function that takes the form data and runs a callback function on the validated user inputs
		//												showInfo: function that takes an HTML payload and displays it to the user, then runs a callback function
		//												label (optional): a human readable label for the WCM item, such as "Wiki"
		//												type (optional): the type of WCM content, such as "WIKI_LIBRARY_TYPE"
		//												submitObj (optional): {
		//													hub (optional): a datastore
		//													item (optional): an item in the datastore
		//												}
		// args MUST contain EITHER label, type OR hub and an item containing the "label", and "type" attributes
		var label = args.label ? args.label : args.submitObj.hub.getValue(args.submitObj.item,"label","");
		label = (args.submitObj.hub && args.submitObj.hub.nlsBundle && args.submitObj.hub.nlsBundle[label]) ? args.submitObj.hub.nlsBundle[label] : label;
		var waitingMsg = args.submitObj.hub.getValue(args.submitObj.item,"waitingMsg","");
		waitingMsg = (args.submitObj.hub && args.submitObj.hub.nlsBundle && args.submitObj.hub.nlsBundle[waitingMsg]) ? args.submitObj.hub.nlsBundle[waitingMsg] : waitingMsg;
		var type = args.type ? args.type : args.submitObj.hub.getValue(args.submitObj.item,"type","");
		var bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
		var genericAddStr = dojo.string.substitute(bundle["new_generic_add"], [label]);
		var processInputs = dojo.hitch(args.controller, function(inputs){ 
			// context "this" object points to controller, args.controller 
			var name = dojo.string.trim(inputs["new_wcm_name"]).replace(/<(?:\/?)script>/g,"");
			if(name && name.length > 0){
				var contr = com.ibm.pb.customize.enabler.AddContentController;
				var closeOverlay = args.showInfo(waitingMsg);
				var validateNameCallback = dojo.hitch(this, function(response){
					closeOverlay();
					if(response == "false"){
						// set up the portlet to get created now
						args.submit(dojo.mixin(args.submitObj,{id:ibmPortalConfig.contentRenderers.wcm,label:name}));
						prefsCallback = dojo.hitch(this, function(data){
							this._addPreferences(this.pendingItems[name], this.getJSRWcmRenderingPreferences, {contextid:data, title:name});
						});
						this.addBeforeSubmit((new dojo.Deferred()).addCallback(dojo.hitch(this, "_processCreateLibrary", name, type, prefsCallback, null)));
					} else {
						// alert the user that the name is already in use
						args.getInfo(form,genericAddStr,bundle["add_content_add"],bundle["new_wcm_wcmNameTaken"],processInputs);
					}
				});
				contr._validateNameField(name, type, validateNameCallback, closeOverlay);
			} else {
				// alert the user that the name is invalid
				args.getInfo(form,genericAddStr,bundle["add_content_add"],bundle["new_wcm_wcmNameInvalid"],processInputs);
			}
		});
		// get a name for the WCM object from the user
	    var form = [{label:bundle["new_wcm_name"],id:"new_wcm_name",value:"",regex:/^[\s\S]/,error:bundle["new_wcm_wcmNameInvalid"]}];
		args.getInfo(form,genericAddStr,bundle["add_content_add"],null,processInputs);
	},

/*******************************************************/
/*       functions to create control preferences   	   */
/*******************************************************/	

	getFeedReaderPreferences: function(/*Object*/itemMap){
		var map = {};
		map.feedURL = itemMap.url;
		map.title = itemMap.label;
		return map;
	},
	
	getChartPreferences: function(/*Object*/itemMap){
		var map = {};
		map.dataSourceURL = itemMap.url;
		map.caption = itemMap.label;
		return map;
	},
	
	getDataViewerPreferences: function(/*Object*/itemMap){
		var map = {};
		map.feedURL = itemMap.url;
		map.tableTitle = itemMap.label;
		return map;
	},
    
	getDrilldownPreferences: function(/*Object*/itemMap){
		var map = {};
        var url={"baseURL":itemMap.url,"parameters":[]};
        
		map.__vUrl = dojo.toJson(url);
		map.tableTitle = itemMap.label;
		return map;
	},    

	getFeedspacePreferences: function(/*Object*/itemMap){
		// summary: get feedspace portlet preferences
	    var url = itemMap.url.replace(/&/g,"&amp;").replace(/>/g,"&gt;").replace(/</g,"&lt;").replace(/"/g,"&quot;").replace(/'/,"&#39;");
		var label = itemMap.label.replace(/&/g,"&amp;").replace(/>/g,"&gt;").replace(/</g,"&lt;").replace(/"/g,"&quot;").replace(/'/,"&#39;");
		var map = {};          
		map.selection = "/0";            
	    var subscriptions = '<?xml version="1.0" encoding="utf-8"?><opml version="2.0"><head><title><![CDATA[My Subscription List]]></title><dateModified><![CDATA[' + Date() + ']]></dateModified></head><body><outline xml:id="Subscription_1';
	    subscriptions = subscriptions + '" text="' + label + '" type="rss" isComment="false" isBreakpoint="false" created="' + Date() + '" category="" title="' + label + '" xmlUrl="' + url + '" /></body></opml>';
	    map.subscriptions = subscriptions;
	    return map;
	},
	
	getJSRWcmRenderingPreferences: function(/*Object*/args){  
		// summary: get WCM rendering portlet preferences
		var displayTitle = args.title.replace(/&/g,"&amp;").replace(/>/g,"&gt;").replace(/</g,"&lt;").replace(/"/g,"&quot;").replace(/'/,"&#39;");
		var map = {};
	    map.WCM_CONTENT_CONTEXT_IDR = args.contextid; 
	    map.WCM_BROADCASTS_TO = "WCM_LINKING_NONE";
	    map.WCM_CONTENT_TYPE = "CONTENT";
	    map.WCM_CONTENT_CONTEXT_TYPE = "sitearea";
	    map.WCM_LISTENS_TO = "WCM_LINKING_OTHER";
	    map.WCM_PORTLET_TITLE = displayTitle;
		map.WCM_PORTLET_TITLE_TYPE = "WCM_PORTLET_TITLE_TYPE_GENERAL";      
	    return map;
	},

/*******************************************************/
/*       Helper functions for WCM  	        		   */
/*******************************************************/

	_pageHasContentMapping: function(/*String*/pageId){
		// get the content mapping url
		var navModel = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel();
		if (!pageId) {
			pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		}
		var page = navModel.find(pageId).start();
		var content = page.getContent().start();
		mappingUrl = com.ibm.mashups.enabler.xml.XPath.evaluateString(
			"//atom:link[@portal:rel='contentmapping']/@href", 
			content.getXml(), 
			{ atom: "http://www.w3.org/2005/Atom", portal: "http://www.ibm.com/xmlns/prod/websphere/portal/v6.0.1/portal-model" }
		);
		// follow the content mapping url
		var ret = false;
		dojo.xhrGet({
			url: mappingUrl,
			handleAs: "xml",
			sync: true,
			load: function(doc){
				// look for the web content mapping entry in the xml
				ret = com.ibm.mashups.enabler.xml.XPath.evaluateBoolean(
					"//contentmapping:content-mapping[((not(@scope)) or (@scope='ibm.wcm') or (@scope='default'))]", 
					doc, 
					{ atom: "http://www.w3.org/2005/Atom", contentmapping: "http://www.ibm.com/xmlns/prod/content-mappings/v1.0" }
				);
			},
			error: function(error){
				console.error(error);
			}
		});
		return ret;
	},

	_addResourceListFeed: function(/*string*/new_name, /*string*/wcmLibraryType, /*string*/operation_id, /*function*/callback, /*function*/ errorFcn){
		// summary: verifies the name or copies a library with the feed
		var baseUrl = ibmPortalConfig.contentHandlerURI;
		var feedUrl = baseUrl + "?uri=wcmbw:id:" + operation_id;
		var message = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf")["new_wcm_wcmNotCopied"];
		var getResourceHandler = dojo.xhrPost({
			url: feedUrl,
			content: {
				blogname: new_name,
				resourcetype: wcmLibraryType,
				pageid: com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID()
			},
			handleAs: "json",
			preventCache: true,
			load: function(data){
				var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID(),
					navModel = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel(),
					page = navModel.find(pageId).start(),
					hasScopeMetadata = page.getMetaData("param.sharing.scope") != null,
					canSet = page.confirmSetMetaData("param.sharing.scope");
				if(operation_id == "copylibrary" && canSet && !hasScopeMetadata){
					page.setMetaData("param.sharing.scope","ibm.wcm.0." + new Date().getTime());
				}
				if(callback) { callback(data.items[0]); }
			},
			error: function(error,ioArgs){
				if(errorFcn) { errorFcn(); }
			}
		 });
		return getResourceHandler;
	},

	_validateNameField: function(/*String*/newName, /*String*/wcmLibraryType, /*function*/callback, /*function*/ errorFcn){
		// summary: makes sure that a name is not already taken in WCM
		var rv = false;
		if (newName && newName.length > 0) {
			rv = true;
			return com.ibm.pb.customize.enabler.AddContentController._addResourceListFeed(newName, wcmLibraryType, "validinput", callback, errorFcn); 
		}
	},

	_processCreateLibrary: function(/*String*/name, /*String*/wcmLibraryType, /*function*/callback, /*function*/ errorFcn){
		// summary: creates a blog/wiki
		var newLibrId = null;
		var templateName = "Wiki Template v70";
		if (wcmLibraryType == "BLOG_LIBRARY_TYPE") {
			templateName = "Blog Template v70";
		}else if (wcmLibraryType == "BLOG_SOLO_TYPE") {
			templateName = "Blog Solo Template v70";
		}    
		return com.ibm.pb.customize.enabler.AddContentController._addResourceListFeed(name, templateName, "copylibrary", callback, errorFcn);
	}
};

dojo.provide("com.ibm.pb.customize.Identifiable");
dojo.declare("com.ibm.pb.customize.Identifiable",com.ibm.mashups.enabler.Identifiable,{
	_id: null,
	
	constructor: function(id) {
		this._id = id;
	},
	
	getID: function() {
		return this._id;
	}
});
