dojo.provide("com.ibm.pb.customize.enabler.ChangeLayoutController");
dojo.require("com.ibm.pb.data.LayoutTemplateModelStore");
com.ibm.pb.customize.enabler.ChangeLayoutController = {
	current: "",		// the layout currently selected on the shelf
	currentSet: "",			// the layout currently set on the page
	url: "",			// url to the current layout
	navModel: null,		// enabler navigation model
	dirty: false,		// true if there are changes pending
	started: false,		// true if the widget is started up
	refresh: true,		// true if a page refresh is required to show saved changes
	layoutMarkupMetadataKey: "com.ibm.portal.layout.template.markup",
	layoutNameMetadataKey: "com.ibm.portal.layout.template.file.name.",
	layoutFileName: "layout.html",
	ibmMainContainer: "ibmMainContainer",
	ibmHiddenWidgets: "ibmHiddenWidgets",
	containerNameMetadataKey: "com.ibm.portal.layoutnode.localname",
		
	startup: function(){
		// summary: sets up variables and UI
		this.logger.entering("startup");
		this.current = this.currentSet;		// the currently selected layout is one set for the page
		this.started = true;			// this widget is now started up
		this.setupUI();
		if(!this.navModel) { this.navModel = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel(); }
		this.logger.exiting("startup");
	},
		
	makeDirty: function(/*bool*/change){
		// summary: changes the dirty attribute
		// change: value to assign to the dirty attribute
		this.logger.entering("makeDirty",[change]);
		this.dirty = change;
		this.logger.exiting("makeDirty",[change]);
	},
		
	processChange: function(/*Object*/args){
		// summary: used as an event handler for when a user clicks on an item
		// args: {id:id of the item clicked}
		this.logger.entering("processChange",[args]);
		this.apply(args); 
		this.logger.exiting("processChange",[args]);
	},
		
	apply: function(/*Object*/args){
		// summary: applies a change associated with a selected item to the page
		// args: {id:id of the item clicked}
		this.logger.entering("apply",[args]);
		if(args.id != this.current) {								// return if no change has been made from the current selection
			if(args.id == this.currentSet) { this.makeDirty(false); }		// Change Layout is no longer dirty if the selection is the same as on the server
			else if(this.dirty === false) { this.makeDirty(true); }	// else make Change Layout dirty if it is not already
			this.current = args.id;	
			this.url = args.url;
		}
		com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(true);
		this.logger.exiting("apply",[args]);
	},
		
	undo: function(){
		// summary: reverts any changes made
		this.logger.entering("undo");
		if(this.started) { this.apply({id:this.currentSet}); }
		this.logger.exiting("undo");
	},
		
	beforeSubmit: function(/*Object*/args){
		// summary: Called before submit and returns a Deferred object which should be used
		// 		to get notified when the controller is ready to be submitted.  Attach a callback
		//		to this Deferred to call the controller's submit function instead of calling
		//		it directly.
		this.logger.entering("beforeSubmit",[args]);
		var dfd = new dojo.Deferred();
		dfd.callback();
		this.logger.exiting("beforeSubmit",[args]);
		return dfd;
	},
		
	submit: function(/*Object*/args){
		// summary: saves any pending changes
		// args: {onComplete:function to call when submission is done, onError:function to call if submission fails}
		this.logger.entering("submit",[args]);
		if(!this.dirty) { 
			// if Change Layout is not dirty, call the onComplete handler and return
			if(args.onComplete) { args.onComplete(); }
			this.logger.exiting("submit",[args]);
			return; 
		}
		var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		if(pageId){
			var pageRef = this.navModel.find(pageId);
			if(pageRef){
				// get the current page and then call setLayout
				pageRef.setFinishedCallback(dojo.hitch(this, "_setLayout"), args);
				pageRef.start(false);
			} else {
				this.logger.severe("submit","Cannot get page node object from Navigation Model",[args]);
				if(args.onError) { args.onError(); }
			}
		} else {
			this.logger.severe("submit","Cannot get current page id",[args]);
			if(args.onError) { args.onError(); }
		}
		this.logger.exiting("submit",[args]);
	},
	
	/**********************************************/
	/*                                            */
	/*                 	helpers                   */
	/*                                            */
	/**********************************************/
	
	_setLayout: function(page, status, args){
		// summary: receives the current page and from that gets the page content
		// page: current page object
		// status: HTTP status returned from request to retrieve page
		// args: from submit()
		this.logger.entering("_setLayout",[page,status,args]);
		if(status == "200"){
			var contentDef = page.getContent();
			if(contentDef){
				contentDef.setFinishedCallback(dojo.hitch(this,"_setMetadata"), args);
				contentDef.start(false);
			} else {
				this.logger.severe("submit","Cannot get page content object",[args]);
				if(args.onError) { args.onError(); }
			}
		} else {
			this.logger.severe("submit","Cannot get page object for current node",[args]);
			if(args.onError) { args.onError(); }
		}
		this.logger.exiting("_setLayout",[page,status,args]);
	},
	
	_setMetadata: function(content, status, args){
		// summary: receives the current page content and from that sets the layout metadata
		// page: current page content object
		// status: HTTP status returned from request to retrieve page content
		// args: from submit()
		this.logger.entering("_setMetadata",[content,status,args]);
		if(status == 200){
			var markup = content.getMetaData(this.layoutMarkupMetadataKey) || "html";
			if(content.confirmSetStaticContentURL(this.url) && content.confirmSetMetaData(this.layoutNameMetadataKey+markup,this.layoutFileName) && content.confirmRefreshStaticMarkup()) {
					content.setStaticContentURL(this.url);
					content.setMetaData(this.layoutNameMetadataKey+markup,this.layoutFileName);
					content.refreshStaticMarkup();
					this.makeDirty(false);		// make Change Layout not dirty
					this.currentSet = this.current; 	// the set layout is now the same as the current
					this._reparentControls(args.onComplete);
			} else {
				this.logger.severe("_setMetadata","User does not have permission to set new static layout template",[content,status,args]);
				if(args.onError) { args.onError(); }
			}
		} else {
			this.logger.severe("_setMetadata","Cannot get page content object",[content,status,args]);
			if(args.onError) { args.onError(); }
		}
		this.logger.exiting("_setMetadata",[content,status,args]);
	},
	
	_reparentControls: function(onComplete){
		// summary: finds the root layout node
		// onComplete: function to call when finished
		this.logger.entering("_reparentControls",[onComplete]);
		var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID(),
			layoutModel = this.navModel.getLayoutModel(pageId),
			rootDef = layoutModel.getRoot();
		if(rootDef) {
			rootDef.setFinishedCallback(dojo.hitch(this, "_processControls"), {layoutModel:layoutModel, onComplete:onComplete});
			rootDef.start(false);
		} else { 
			this.logger.severe("_reparentControls","Cannot find root layout container in the Layout Model",[onComplete]);  
			if(onComplete) { onComplete(); }
		}
		this.logger.exiting("_reparentControls",[onComplete]);
	},
	
	_processControls: function(root, status, x){
		// summary: gets the children of the root layout node and reparents those in containers to be deleted
		// root: root layout node object
		// status: HTTP status from request to retrieve root
		// x.layoutModel: layout model for the page
		// x.onComplete: function to call when finished
		this.logger.entering("_processControls",[root,status,x]);
		if(status == 200 || status == "200"){
			var children = x.layoutModel.getChildren(root);
			var newParent = null,
				orphans = [];
			// find the named main layout container
			while(children.hasNext()){
				var child = children.next();
				// found main container
				if(child.getMetaData(this.containerNameMetadataKey) == this.ibmMainContainer) { newParent = child; }
				// skip hidden widgets container
				else if (child.getMetadata(this.containerNameMetadataKey) == this.ibmHiddenWidgets) { continue; }
				else { // reparent controls from this container into the main container
					var grandChildren = x.layoutModel.getChildren(child);
					while(grandChildren.hasNext()) { orphans.push(grandChildren.next()); }
				}
			}
			if(newParent) {
				// reparent the orphans into newParent
				for(var i = 0; i < orphans.length; i++){
					if(!(orphans[i] instanceof com.ibm.mashups.enabler.layout.LayoutControl)) { this.logger.warning("_reparentControls","Orphan is not an instance of a layout control.  Skipping...");  continue; }
					/*if(x.layoutModel.confirmInsert(orphans[i], newParent)) { */try{x.layoutModel.insert(orphans[i], newParent); }catch(e){console.error(e);}//}
					//else { this.logger.warning("_reparentControls","Cannot insert orphaned control into new container.  It will be lost.");  continue; }
				}
			} else { this.logger.warning("_reparentControls","Cannot find the container named '" + this.ibmMainContainer + "' to hold orphaned controls.  They will be lost."); }
			if(x.onComplete) { x.onComplete(); }
		} else {
			this.logger.severe("_processControls","Cannot find root layout container in the Layout Model",[root,status,x]); 
			if(x.onComplete) { x.onComplete(); }
		}
		this.logger.exiting("_processControls",[root,status,x]);
	}
};
