dojo.provide("com.ibm.pb.customize.enabler.ChangeStyleController");
dojo.require("com.ibm.pb.layout.LayoutParser");
com.ibm.pb.customize.enabler.ChangeStyleController = {
	current: "",		// the style currently selected on the shelf
	currentSet: "",			// the style currently set on the page
	url: "",			// url to the current stylesheet
	navModel: null,		// enabler navigation model
	dirty: false,		// true if there are changes pending
	started: false,		// true if the widget is started up
	refresh: false,		// true if a page refresh is required to show saved changes
	styleMetadataKey: "colorPalette",
	urlPrefix: ibmCfg.themeConfig.themeRootURI + "/",
	layoutParser: com.ibm.pb.layout.LayoutParser,
	
	startup: function(){
		// summary: sets up variables and UI
		this.logger.entering("startup");
		this.current = this.currentSet;			// the currently selected style is one set for the page
		this.started = true;				// this widget is now started up
		this.setupUI();						// set up the Change Style UI
		if(!this.navModel) { this.navModel = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel(); }
		this.logger.exiting("startup");
	},
		
	makeDirty: function(/*bool*/change){
		// summary: changes the dirty attribute
		// change: value to assign to the dirty attribute
		this.logger.entering("makeDirty",[change]);
		this.dirty = change;
		this.logger.exiting("makeDirty",[change]);
	},
		
	processChange: function(/*Object*/args){
		// summary: used as an event handler for when a user clicks on an item
		// args: {id:id of the item clicked}
		this.logger.entering("processChange",[args]);
		this.apply(args);
		this.logger.exiting("processChange",[args]);
	},
		
	apply: function(/*Object*/args){
		// summary: applies a change associated with a selected item to the page
		// args: {id:id of the item clicked}
		this.logger.entering("apply",[args]);
		if(args.id != this.current) {								// return if no change has been made from the current selection
			if(args.id == this.currentSet) { this.makeDirty(false); }		// Change Style is no longer dirty if the selection is the same as on the server
			else if(this.dirty === false) { this.makeDirty(true); }	// else make Change Style dirty if it is not already
			this.current = args.id;									// set current to the given id
			this.url = args.url;									// set the current url
			this._switchStylesheet(this.current,this.url);			// change the stylesheet on the client
		}
		com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(true);
		this.logger.exiting("apply",[args]);
	},
		
	undo: function(){
		// summary: reverts any changes made
		this.logger.entering("undo");
		if(this.started) { this.apply({id:this.currentSet}); }
		this.logger.exiting("undo");
	},
		
	beforeSubmit: function(/*Object*/args){
		// summary: Called before submit and returns a Deferred object which should be used
		// 		to get notified when the controller is ready to be submitted.  Attach a callback
		//		to this Deferred to call the controller's submit function instead of calling
		//		it directly.
		this.logger.entering("beforeSubmit",[args]);
		var dfd = new dojo.Deferred();
		dfd.callback();
		this.logger.exiting("beforeSubmit",[args]);
		return dfd;
	},
		
	submit: function(/*Object*/args){
		// summary: saves any pending changes
		// args: {onComplete:function to call when submission is done, onError:function to call if submission fails}
		this.logger.entering("submit",[args]);
		if(!this.dirty) { 
			// if Change Style is not dirty, call the onComplete handler and return
			if(args.onComplete) { args.onComplete(); }
			this.logger.exiting("submit",[args]);
			return; 
		}
		var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		if(pageId) {
			var pageRef = this.navModel.find(pageId);
			if(pageRef){
				pageRef.setFinishedCallback(dojo.hitch(this, "_setMetadata"), args);
				pageRef.start(false);
			} else {
				this.logger.severe("submit","Cannot get page node object from Navigation Model",[args]);
				if(args.onError) { args.onError(); }
			}
		} else {
			this.logger.severe("submit","Cannot get current page id",[args]);
			if(args.onError) { args.onError(); }
		}	
		this.logger.exiting("submit",[args]);
	},
		
	/***********************************************/
	/*              Helper functions   		       */
	/***********************************************/

	_setMetadata: function(page, status, args){
		// summary: receives the current page and from that sets the layout metadata
		// page: current page object
		// status: HTTP status returned from request to retrieve page
		// args: from submit()
		this.logger.entering("_setMetadata",[page,status,args]); 
		if(status == "200") {
			var content = page.getContent().start();
			if(this.url && content.confirmSetMetaData(this.styleMetadataKey,this.url)){
				if(this.url) content.setMetaData(this.styleMetadataKey,this.url);	// set the style metadata to the currently selected
			} else if(!this.url && content.confirmRemoveMetaData(this.styleMetadataKey)){
				content.removeMetaData(this.styleMetadataKey);
			} else {
				this.logger.severe("submit","User does not have permission to set or delete page metadata",[args]);
				if(args.onError) { args.onError(); }
			}
			this.makeDirty(false);		// make Change Style not dirty
			this.currentSet = this.current; 	// the set style is now the same as the current
			if(args.onComplete) { args.onComplete(); }
		} else {
			this.logger.severe("submit","Cannot find the page object for the current node",[args]);
			if(args.onError) { args.onError(); }
		}
		this.logger.exiting("_setMetadata",[page,status,args]);
	},
	
	_switchStylesheet: function(title,url){
		// summary: switch to the stylesheet with the given title
		var i, currLink, found = false;
		for(i = 0; (currLink = document.getElementsByTagName("link")[i]); i++) {
			// if the link is a perferred stylesheet
			if(currLink.getAttribute("rel").indexOf("style") >= 0 && currLink.getAttribute("title")) { 
				// if the chosen theme is found
				if(currLink.getAttribute("title") == title) {
					currLink.disabled = false;
					if(dojo.isWebKit){
						// workaround a WebKit bug that doesn't enable an alt stylesheet on the first try
						currLink.disabled = true;
						currLink.disabled = false;
					}
					found = true;
				}
				else { currLink.disabled = true;  }
			}
		}
		if(dojo.isWebKit) { 
			// workaround the Safari 3.1 and Chrome bug where an alternate stylesheet can never be turned on via js
			// https://discussions.apple.com/thread/3215084?start=0&tstart=0
			if(found) {
				if(!this.webkitLink) {
					this.webkitLink = dojo.create("link",{href:this.urlPrefix+url,type:"text/css",rel:"stylesheet"});
					document.getElementsByTagName("head")[0].appendChild(this.webkitLink);
				} else {
					this.webkitLink.href = this.urlPrefix + url;
				}
			} else if (!url && this.webkitLink) {
				document.getElementsByTagName("head")[0].removeChild(this.webkitLink);
				this.webkitLink = null;
			}
		}
		if(!found && url) {
			this._loadCss(url,title,true,true); 
			this._switchStylesheet(title);
		}
	},

	_loadCss: function(path, title, alternate, disabled){
		if(dojo.isIE){
			var link = document.createElement("link");
			link.setAttribute("disabled",disabled);
			if(title) { link.title = title; }
			var head = document.getElementsByTagName("head")[0];
			head.appendChild(link);
			link.setAttribute("type","text/css");
			link.setAttribute("rel",(alternate)?"alternate stylesheet":"stylesheet");
			link.setAttribute("href",this.urlPrefix+path);
			link.setAttribute("media","screen");
		} else {
			var link = dojo.create("link",{type:"text/css",rel:(alternate)?"alternate stylesheet":"stylesheet",href:this.urlPrefix+path,media:"screen",disabled:disabled});
			if(title) { link.title = title; }
			var head = document.getElementsByTagName("head")[0];
			head.appendChild(link);
		}
	}
};
