-- *****************************************************************
--
-- Licensed Materials - Property of IBM
--                                      
-- 5724-U69                             
--                                      
-- Copyright IBM Corp. 2001, 2006  All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with
-- IBM Corp.
--
-- *****************************************************************

-- Mashup Derby Script for creating tables, keys, indexes

-- *****************************************************************
--
-- SCHEMA VERSION (100)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.VERSION_INFO(
    OID VARCHAR(64)    NOT NULL,
    VERSION_MAJOR   INTEGER,
    VERSION_MINOR   INTEGER,
    VERSION_RELEASE INTEGER,
    VERSION_DEV     INTEGER,
    VERSION_TIME    BIGINT,
    DESCRIPTION     VARCHAR(254),
    CONSTRAINT PK100 PRIMARY KEY (OID)
);

-- *****************************************************************
--
-- ACCOUNT (200)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.ACCOUNT (
    OID         VARCHAR(64)        NOT NULL,
    EXTERNAL_ID VARCHAR(1024)      NOT NULL,
    CREATED     BIGINT             NOT NULL,
    MODIFIED    BIGINT             NOT NULL,
    STATE       INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK200 PRIMARY KEY (OID)
);

-- *****************************************************************
--
-- CATALOG (300)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.CATALOG (
    OID         VARCHAR(64)       NOT NULL,
    ACCOUNT_OID VARCHAR(64)       NOT NULL,
    UNIQUE_NAME VARCHAR(255)      NOT NULL,
    CREATED     BIGINT            NOT NULL,
    MODIFIED    BIGINT            NOT NULL,
    STATE       INTEGER DEFAULT 1 NOT NULL,
    CONSTRAINT PK300 PRIMARY KEY (OID)
);

CREATE TABLE @SCHEMA@.CATALOG_LOD (
    RESOURCE_OID VARCHAR(64)       NOT NULL,
    LOCALE       VARCHAR(64)    NOT NULL,
    TITLE        VARCHAR(255)   NOT NULL,
    DESCRIPTION  VARCHAR(1024)  ,
    CONSTRAINT PK301 PRIMARY KEY (RESOURCE_OID, LOCALE),
    CONSTRAINT FK302 FOREIGN KEY (RESOURCE_OID) REFERENCES @SCHEMA@.CATALOG (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- CATALOG ENTRY (310)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.CATALOG_ENTRY (
    OID            			VARCHAR(64)        NOT NULL,
    CATALOG_OID    			VARCHAR(64)        NOT NULL,
    CATEGORY_OID   			VARCHAR(64)        NOT NULL,
    DEFINITION_URL 			VARCHAR(2500)      NOT NULL,
    CONTENT_URL    			VARCHAR(2500)      ,
    PREVIEW_URL    			VARCHAR(2500)      ,
    PREVIEW_THUMBNAIL_URL	VARCHAR(2500)      ,
    HELP_URL       			VARCHAR(2500)      ,
    ICON_URL       			VARCHAR(2500)      ,
    HUB_ENTRY_URL  			VARCHAR(2500)      ,
    DEF_URL_TYPE   			INTEGER            ,
    ICON_URL_TYPE  			INTEGER            ,
    CREATED        			BIGINT             NOT NULL,
    MODIFIED       			BIGINT             NOT NULL,
    STATE          			INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK310 PRIMARY KEY (OID)
);

CREATE TABLE @SCHEMA@.CATALOG_ENTRY_LOD (
    RESOURCE_OID  		VARCHAR(64)    NOT NULL,
    LOCALE        		VARCHAR(64)    NOT NULL,
    TITLE         		VARCHAR(255)   NOT NULL,
    DESCRIPTION   		VARCHAR(1024)  ,
    SHORT_DESCRIPTION   VARCHAR(255)   ,
    CONSTRAINT PK311 PRIMARY KEY (RESOURCE_OID, LOCALE),
    CONSTRAINT FK311 FOREIGN KEY (RESOURCE_OID) REFERENCES @SCHEMA@.CATALOG_ENTRY (OID) ON DELETE CASCADE
);

CREATE TABLE @SCHEMA@.CATALOG_ENTRY_DD (
    RESOURCE_OID  VARCHAR(64)    NOT NULL,
    NAME          VARCHAR(255)   NOT NULL,
    VALUE         VARCHAR(3000)  NOT NULL,
    CONSTRAINT PK312 PRIMARY KEY (RESOURCE_OID, NAME),
    CONSTRAINT FK312 FOREIGN KEY (RESOURCE_OID) REFERENCES @SCHEMA@.CATALOG_ENTRY (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- CATALOG CATEGORY (320)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.CATALOG_CATEGORY (
    OID         VARCHAR(64)         NOT NULL,
    CATALOG_OID VARCHAR(64)         NOT NULL,
    NAME        VARCHAR(255)        ,
    CREATED     BIGINT              NOT NULL,
    MODIFIED    BIGINT              NOT NULL,
    STATE       INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK320 PRIMARY KEY (OID),
    CONSTRAINT FK320 FOREIGN KEY (CATALOG_OID) REFERENCES @SCHEMA@.CATALOG (OID) ON DELETE CASCADE
);

CREATE TABLE @SCHEMA@.CATALOG_CATEGORY_LOD (
    RESOURCE_OID VARCHAR(64)    NOT NULL,
    LOCALE       VARCHAR(64)    NOT NULL,
    TITLE        VARCHAR(255)   NOT NULL,
    DESCRIPTION  VARCHAR(1024)  ,
    CONSTRAINT PK321 PRIMARY KEY (RESOURCE_OID, LOCALE),
    CONSTRAINT FK321 FOREIGN KEY (RESOURCE_OID) REFERENCES @SCHEMA@.CATALOG_CATEGORY (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- CATALOG CATEGORY INCLUDES (330)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.CATALOG_CATEGORY_INC (
    OID          		VARCHAR(64)       NOT NULL,
    CATALOG_OID  		VARCHAR(64)       NOT NULL,
    CATEGORY_OID 		VARCHAR(64)       NOT NULL,
    INCLUDE_CAT  		VARCHAR(255)  	  NOT NULL,
    INCLUDE_CATEGORY	VARCHAR(255)      NOT NULL,
    CONSTRAINT PK330 PRIMARY KEY (OID)
);

-- *****************************************************************
--
-- CATALOG INCLUDES (340)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.CATALOG_INC (
    OID          		VARCHAR(64)       NOT NULL,
    CATALOG_OID  		VARCHAR(64)       NOT NULL,
    INCLUDE_CAT  		VARCHAR(255)  	  NOT NULL,
    CONSTRAINT PK340 PRIMARY KEY (OID)
);

-- *****************************************************************
--
-- NAVIGATION NODE (500)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.NAVNODE (
    OID            VARCHAR(64)        NOT NULL,
    OWNER_OID      VARCHAR(64)        NOT NULL,
    THEME_ID       VARCHAR(255)       NOT NULL,
    THEME_NAME     VARCHAR(255)       NOT NULL,
    CONTENT        BLOB(1M)           ,
    BYTE_COUNT     INTEGER            ,
    IS_ZIPPED      INTEGER DEFAULT 0  NOT NULL,
    COMMUNITY_OID  VARCHAR(64)        ,
    SPACE_OID      VARCHAR(64)        ,
    CREATED        BIGINT             NOT NULL,
    MODIFIED       BIGINT             NOT NULL,
    STATE          INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK500 PRIMARY KEY (OID, OWNER_OID)
);

CREATE TABLE @SCHEMA@.NAVNODE_LOD (
    RESOURCE_OID  VARCHAR(64)    NOT NULL,
    OWNER_OID     VARCHAR(64)    NOT NULL,
    LOCALE        VARCHAR(64)    NOT NULL,
    TITLE         VARCHAR(255)   NOT NULL,
    DESCRIPTION   VARCHAR(1024)          ,
    CONSTRAINT FK500 FOREIGN KEY (RESOURCE_OID,OWNER_OID) REFERENCES @SCHEMA@.NAVNODE (OID,OWNER_OID) ON DELETE CASCADE
);

CREATE TABLE @SCHEMA@.NAVNODE_DD (
    RESOURCE_OID  VARCHAR(64)    NOT NULL,
    OWNER_OID     VARCHAR(64)    NOT NULL,
    NAME          VARCHAR(255)   NOT NULL,
    VALUE         VARCHAR(3000)  NOT NULL,
    CONSTRAINT PK502 PRIMARY KEY (RESOURCE_OID,OWNER_OID,NAME),
    CONSTRAINT FK502 FOREIGN KEY (RESOURCE_OID,OWNER_OID) REFERENCES @SCHEMA@.NAVNODE (OID,OWNER_OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- NAVIGATION TREE (510)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.NAV_TREE (
    OWNER_OID    VARCHAR(64)         NOT NULL,
    PAGE_OID     VARCHAR(64)         NOT NULL,
    PARENT_OID   VARCHAR(64)         NOT NULL,
    ORDINAL      INTEGER  DEFAULT 1  ,
    IS_VIRTUAL   INTEGER  DEFAULT 0  ,
    CONSTRAINT PK510 PRIMARY KEY (OWNER_OID, PAGE_OID)
);

-- *****************************************************************
--
-- ACCESS CONTROL (600)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.COMMUNITY_DEF (
    OID            VARCHAR(64)        NOT NULL,
    OWNER_UID      VARCHAR(64)        NOT NULL,
    RESOURCE_OID   VARCHAR(64)        NOT NULL,
    RESOURCE_TYPE  VARCHAR(32)        NOT NULL,
    CREATED        BIGINT             NOT NULL,
    MODIFIED       BIGINT             NOT NULL,
    STATE          INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK600 PRIMARY KEY( OID )
);
-- UNIQUE CONSTRAINT
ALTER TABLE @SCHEMA@.COMMUNITY_DEF ADD CONSTRAINT UQ600 UNIQUE (OID,RESOURCE_OID);

CREATE TABLE @SCHEMA@.ACL (
    COMMUNITY_OID     VARCHAR(64)        NOT NULL,
    PARTICIPANT_UID   VARCHAR(64)        NOT NULL,
    PARTICIPANT_TYPE  INTEGER            NOT NULL,
    PERMISSIONS       VARCHAR(255)       NOT NULL,
    CREATED           BIGINT             NOT NULL,
    MODIFIED          BIGINT             NOT NULL,
    STATE             INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK610 PRIMARY KEY (COMMUNITY_OID,PARTICIPANT_UID),
    CONSTRAINT FK611 FOREIGN KEY (COMMUNITY_OID) REFERENCES @SCHEMA@.COMMUNITY_DEF (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- SPACES (700)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.SPACENODE (
    OID            VARCHAR(64)        NOT NULL,
    OWNER_OID      VARCHAR(64)        NOT NULL,
    ROOTNAV_OID    VARCHAR(64)        ,
    TEMPLATE_OID   VARCHAR(64)        ,
    CREATED        BIGINT             NOT NULL,
    MODIFIED       BIGINT             NOT NULL,
    STATE          INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK700 PRIMARY KEY (OID, OWNER_OID)
);

CREATE TABLE @SCHEMA@.SPACENODE_LOD (
    RESOURCE_OID  VARCHAR(64)    NOT NULL,
    OWNER_OID     VARCHAR(64)    NOT NULL,
    LOCALE        VARCHAR(64)    NOT NULL,
    TITLE         VARCHAR(255)   NOT NULL,
    DESCRIPTION   VARCHAR(1024)          ,
    CONSTRAINT PK701 PRIMARY KEY (RESOURCE_OID,OWNER_OID,LOCALE),
    CONSTRAINT FK701 FOREIGN KEY (RESOURCE_OID,OWNER_OID) REFERENCES @SCHEMA@.SPACENODE (OID,OWNER_OID) ON DELETE CASCADE
);

CREATE TABLE @SCHEMA@.SPACENODE_DD (
    RESOURCE_OID  VARCHAR(64)    NOT NULL,
    OWNER_OID     VARCHAR(64)    NOT NULL,
    NAME          VARCHAR(255)   NOT NULL,
    VALUE         VARCHAR(3000)  NOT NULL,
    CONSTRAINT PK702 PRIMARY KEY (RESOURCE_OID,OWNER_OID,NAME),
    CONSTRAINT FK702 FOREIGN KEY (RESOURCE_OID,OWNER_OID) REFERENCES @SCHEMA@.SPACENODE (OID,OWNER_OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- TEMPLATES (800)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.TEMPLATENODE (
    OID            VARCHAR(64)        NOT NULL,
    OWNER_OID      VARCHAR(64)        ,
    CONTENT        BLOB(1M)           ,
    BYTE_COUNT     INTEGER            ,
    TEMPLATE_TYPE  VARCHAR(64)        ,
    CREATED        BIGINT             NOT NULL,
    MODIFIED       BIGINT             NOT NULL,
    STATE          INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK800 PRIMARY KEY (OID)
);

CREATE TABLE @SCHEMA@.TEMPLATENODE_LOD (
    RESOURCE_OID  VARCHAR(64)    NOT NULL,
    LOCALE        VARCHAR(64)    NOT NULL,
    TITLE         VARCHAR(255)   NOT NULL,
    DESCRIPTION   VARCHAR(1024)          ,
    CONSTRAINT PK801 PRIMARY KEY (RESOURCE_OID,LOCALE),
    CONSTRAINT FK801 FOREIGN KEY (RESOURCE_OID) REFERENCES @SCHEMA@.TEMPLATENODE (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- ACCESS CONTROL RESOURCES (900)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.AC_RESOURCE (
    OID            VARCHAR(64)        NOT NULL,
    OWNER_UID      VARCHAR(64)        NOT NULL,
    RESOURCE_OID   VARCHAR(64)        NOT NULL,
    RESOURCE_TYPE  VARCHAR(32)        NOT NULL,
    ROLE_BLOCKS    VARCHAR(1024)              ,
    CREATED        BIGINT             NOT NULL,
    MODIFIED       BIGINT             NOT NULL,
    STATE          INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK900 PRIMARY KEY( RESOURCE_OID )
);

CREATE TABLE @SCHEMA@.AC_ROLE (
    AC_RESOURCE_OID   VARCHAR(64)        NOT NULL,
    ROLE_OID          VARCHAR(64)        NOT NULL,
    ROLE_NAME         VARCHAR(1024)      NOT NULL,
    ROLE_TYPE         VARCHAR(1024)              ,
    CREATED           BIGINT             NOT NULL,
    MODIFIED          BIGINT             NOT NULL,
    STATE             INTEGER DEFAULT 1  NOT NULL,
    CONSTRAINT PK910 PRIMARY KEY( ROLE_OID ),
    CONSTRAINT FK911 FOREIGN KEY (AC_RESOURCE_OID) REFERENCES @SCHEMA@.AC_RESOURCE (RESOURCE_OID) ON DELETE CASCADE
);

CREATE TABLE @SCHEMA@.AC_MEMBER (
    ROLE_OID          VARCHAR(64)        NOT NULL,
    MEMBER_UID        VARCHAR(64)        NOT NULL,
    MEMBER_TYPE       INTEGER            NOT NULL,
    CONSTRAINT PK920 PRIMARY KEY (ROLE_OID,MEMBER_UID),
    CONSTRAINT FK921 FOREIGN KEY (ROLE_OID) REFERENCES @SCHEMA@.AC_ROLE (ROLE_OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- FILESTORE (1000)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.FILESTORE_PATH (
    OID         VARCHAR(64)           NOT NULL,
    PARENT_OID  VARCHAR(64)           NOT NULL,
    NAME        VARCHAR(255)          NOT NULL,
    PATH        VARCHAR(255)          NOT NULL,
    LOCKED      VARCHAR(64)           , 
    OWNER       VARCHAR(64)           ,             
    CREATED     BIGINT                NOT NULL,
    MODIFIED    BIGINT                NOT NULL,
    COLL_TYPE   INTEGER DEFAULT 1     NOT NULL,
    CONSTRAINT PK1010 PRIMARY KEY (OID)
);

CREATE TABLE @SCHEMA@.FILESTORE_FILE (
    OID         VARCHAR(64)           NOT NULL,      
    F_SIZE      INTEGER               NOT NULL,
    CONTENT     BLOB(1G)           ,
    CONSTRAINT PK1000 PRIMARY KEY (OID),
    CONSTRAINT FK1001 FOREIGN KEY (OID) REFERENCES @SCHEMA@.FILESTORE_PATH (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- LAYOUT NODE (1200)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.LAYOUTNODE (
    OID                VARCHAR(64)        NOT NULL,
    PAGE_OID           VARCHAR(64)        NOT NULL,
    SKIN_OID           VARCHAR(255)               ,
    NODE_TYPE          INTEGER DEFAULT 1  NOT NULL,
    CREATED            BIGINT             NOT NULL,
    MODIFIED           BIGINT             NOT NULL,
    CONSTRAINT PK1200 PRIMARY KEY (OID, PAGE_OID)
);

CREATE TABLE @SCHEMA@.LAYOUTNODE_DD (
    PAGE_OID      VARCHAR(64)    NOT NULL,
    RESOURCE_OID  VARCHAR(64)    NOT NULL,
    NAME          VARCHAR(255)   NOT NULL,
    VALUE         VARCHAR(3000)  NOT NULL,
    CONSTRAINT PK1202 PRIMARY KEY (RESOURCE_OID,PAGE_OID,NAME),
    CONSTRAINT FK1202 FOREIGN KEY (RESOURCE_OID,PAGE_OID) REFERENCES @SCHEMA@.LAYOUTNODE (OID,PAGE_OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- LAYOUT TREE (1210)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.LAYOUT_TREE (
    LAYOUTNODE_OID VARCHAR(64)         NOT NULL,
    PAGE_OID       VARCHAR(64)         NOT NULL,
    PARENT_OID     VARCHAR(64)         NOT NULL,
    ORDINAL        INTEGER  DEFAULT 1  ,
    CONSTRAINT PK1210 PRIMARY KEY (LAYOUTNODE_OID,PAGE_OID),
    CONSTRAINT FK1210 FOREIGN KEY (LAYOUTNODE_OID,PAGE_OID) REFERENCES @SCHEMA@.LAYOUTNODE (OID,PAGE_OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- WIDGET RELATION (1300)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.WIDGET_RELATION (
  OID             VARCHAR(64)          NOT NULL,
  CONSTRAINT PK1300 PRIMARY KEY (OID)
);

-- *****************************************************************
--
-- WIDGET DEFINITION (1310)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.WIDGET_DEFINITION (
  OID             VARCHAR(64)          NOT NULL,
  DEFINITION_URL  VARCHAR(2048)        NOT NULL,
  WIDGET_ID       VARCHAR(255)         NOT NULL,
  WIDGET_TYPE     VARCHAR(20)          NOT NULL,  
  CREATED         BIGINT               NOT NULL,
  MODIFIED        BIGINT               NOT NULL,
  CONSTRAINT PK1310 PRIMARY KEY (OID),
  CONSTRAINT FK1310 FOREIGN KEY (OID) REFERENCES @SCHEMA@.WIDGET_RELATION (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- WIDGET INSTANCE (1320)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.WIDGET_INSTANCE (
  OID        VARCHAR(64)        NOT NULL,
  OWNER_OID  VARCHAR(64)        ,
  PARENT_OID VARCHAR(64)        ,
  SANDBOX    INTEGER DEFAULT 0  NOT NULL,
  CREATED    BIGINT NOT NULL    ,
  MODIFIED   BIGINT NOT NULL    ,
  CONSTRAINT PK1320 PRIMARY KEY (OID),
  CONSTRAINT FK1320 FOREIGN KEY (PARENT_OID) REFERENCES @SCHEMA@.WIDGET_INSTANCE (OID) ON DELETE CASCADE,
  CONSTRAINT FK1321 FOREIGN KEY (OID) REFERENCES @SCHEMA@.WIDGET_RELATION (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- WIDGET WINDOW (1330)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.WIDGET_WINDOW (
  OID            VARCHAR(64)        NOT NULL,
  PAGE_OID       VARCHAR(64)        NOT NULL,
  DEFINITION_OID VARCHAR(64)        NOT NULL,
  INSTANCE_OID   VARCHAR(64)        NOT NULL,
  CREATED        BIGINT             NOT NULL,
  MODIFIED       BIGINT             NOT NULL,
  CONSTRAINT PK1330 PRIMARY KEY (OID),
  CONSTRAINT FK1330 FOREIGN KEY (OID, PAGE_OID) REFERENCES @SCHEMA@.LAYOUTNODE (OID, PAGE_OID) ON DELETE CASCADE,
  CONSTRAINT FK1331 FOREIGN KEY (DEFINITION_OID) REFERENCES @SCHEMA@.WIDGET_DEFINITION (OID) ON DELETE CASCADE,
  CONSTRAINT FK1332 FOREIGN KEY (INSTANCE_OID) REFERENCES @SCHEMA@.WIDGET_INSTANCE (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- WIDGET BLOB (1340)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.WIDGET_WIDGETBLOB (
  PARENT_OID       VARCHAR(64)  NOT NULL,
  CONTENT          BLOB(1M),
  CONSTRAINT PK1340 PRIMARY KEY (PARENT_OID),
  CONSTRAINT FK1340 FOREIGN KEY (PARENT_OID) REFERENCES @SCHEMA@.WIDGET_RELATION (OID) ON DELETE CASCADE
);


-- *****************************************************************
--
-- WIRE (1400)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.WIREITEM (
    OID         VARCHAR(64)           NOT NULL,
    PAGE_OID    VARCHAR(64)           NOT NULL,
    CREATED     BIGINT                NOT NULL,
    MODIFIED    BIGINT                NOT NULL,
    S_WIDGET    VARCHAR(64)           NOT NULL,
    S_EVENT     VARCHAR(255)          NOT NULL,
    S_EVENTNAME VARCHAR(255)                  ,
    S_PAGE      VARCHAR(64)           NOT NULL,
    T_WIDGET    VARCHAR(64)           NOT NULL,
    T_EVENT     VARCHAR(255)          NOT NULL,
    T_EVENTNAME VARCHAR(255)                  ,
    T_PAGE      VARCHAR(64)           NOT NULL,
    CONSTRAINT PK1400 PRIMARY KEY (OID),
    CONSTRAINT FK1400 FOREIGN KEY (S_WIDGET,PAGE_OID) REFERENCES @SCHEMA@.LAYOUTNODE (OID,PAGE_OID) ON DELETE CASCADE,
    CONSTRAINT FK1401 FOREIGN KEY (T_WIDGET,PAGE_OID) REFERENCES @SCHEMA@.LAYOUTNODE (OID,PAGE_OID) ON DELETE CASCADE 
);

-- *****************************************************************
--
-- LAYOUT_MARKUP_NODE (1500)
--
-- *****************************************************************
CREATE TABLE @SCHEMA@.LAYOUT_MARKUP_NODE (
	PAGE_OID    VARCHAR(64)           NOT NULL,
	CREATED     BIGINT                NOT NULL,
	MODIFIED    BIGINT                NOT NULL,
	CONTENT     BLOB(3M),
 	CONSTRAINT PK1500 PRIMARY KEY (PAGE_OID)
);

--*****************************************************************
--
-- Indexing
--
--*****************************************************************
-- Catalog
CREATE INDEX @SCHEMA@.CATACCT      ON @SCHEMA@.CATALOG(ACCOUNT_OID);
CREATE INDEX @SCHEMA@.ENTRYCATGOID ON @SCHEMA@.CATALOG_ENTRY(CATALOG_OID, CATEGORY_OID);
CREATE INDEX @SCHEMA@.CATGCATOID   ON @SCHEMA@.CATALOG_CATEGORY(CATALOG_OID);
CREATE INDEX @SCHEMA@.CATINTCATOID ON @SCHEMA@.CATALOG_INC(CATALOG_OID);

-- Navigation
CREATE INDEX @SCHEMA@.PAGECOMM     ON @SCHEMA@.NAVNODE(COMMUNITY_OID);
CREATE INDEX @SCHEMA@.PAGELODOWNRS ON @SCHEMA@.NAVNODE_LOD(OWNER_OID,RESOURCE_OID);
CREATE INDEX @SCHEMA@.TREEOWPAR    ON @SCHEMA@.NAV_TREE(OWNER_OID,PARENT_OID);

-- Acl
CREATE INDEX @SCHEMA@.COMMOWNPAR ON @SCHEMA@.COMMUNITY_DEF(OWNER_UID);
CREATE INDEX @SCHEMA@.PRTTYUID   ON @SCHEMA@.ACL(PARTICIPANT_TYPE,PARTICIPANT_UID);
CREATE INDEX @SCHEMA@.PRTUID     ON @SCHEMA@.ACL(PARTICIPANT_UID);

-- AC
CREATE INDEX @SCHEMA@.AC_ROLE_RESOURCE ON @SCHEMA@.AC_ROLE(AC_RESOURCE_OID);
CREATE INDEX @SCHEMA@.AC_MEMBER_UID ON @SCHEMA@.AC_MEMBER(MEMBER_UID);
CREATE INDEX @SCHEMA@.AC_MEMBER_MEMBERTYPE ON @SCHEMA@.AC_MEMBER(MEMBER_TYPE ASC);
CREATE INDEX @SCHEMA@.AC_RESOURCE_OWNER ON @SCHEMA@.AC_RESOURCE(OWNER_UID,RESOURCE_OID);

-- FILESTORE_PATH
CREATE INDEX @SCHEMA@.FILESTORE_PATH_1 ON @SCHEMA@.FILESTORE_PATH( PATH );
CREATE INDEX @SCHEMA@.FILESTORE_PATH_2 ON @SCHEMA@.FILESTORE_PATH( PARENT_OID);

-- Layout
CREATE INDEX @SCHEMA@.LAYOUT_TREE_1 ON @SCHEMA@.LAYOUT_TREE(PAGE_OID ASC, PARENT_OID DESC);

-- Account
CREATE INDEX @SCHEMA@.ACCOUNT_1 ON @SCHEMA@.ACCOUNT(EXTERNAL_ID);

-- Widget & Wire
CREATE INDEX @SCHEMA@.WIREITEM_PAGEOID ON @SCHEMA@.WIREITEM(PAGE_OID);
CREATE INDEX @SCHEMA@.WIREITEM_SEVENT ON @SCHEMA@.WIREITEM(S_EVENT);
CREATE INDEX @SCHEMA@.WIREITEM_TEVENT ON @SCHEMA@.WIREITEM(T_EVENT);
CREATE INDEX @SCHEMA@.WIDGETDEFINITION_DEFINITIONURL ON @SCHEMA@.WIDGET_DEFINITION(DEFINITION_URL);
CREATE INDEX @SCHEMA@.WIDGETDEFINITION_WIDGETID ON @SCHEMA@.WIDGET_DEFINITION(WIDGET_ID);
CREATE INDEX @SCHEMA@.WIDGETINSTANCE_PARENTOID ON @SCHEMA@.WIDGET_INSTANCE(PARENT_OID);
CREATE INDEX @SCHEMA@.WIDGETINSTANCE_OWNEROID ON @SCHEMA@.WIDGET_INSTANCE(OWNER_OID);
CREATE INDEX @SCHEMA@.WIDGETWINDOW_INSTANCEOID ON @SCHEMA@.WIDGET_WINDOW(INSTANCE_OID);

--*****************************************************************
--
-- Initialization
--
--*****************************************************************
-- Schema version (leave this here to faciliate version checking)

INSERT INTO @SCHEMA@.VERSION_INFO (OID,VERSION_MAJOR,VERSION_MINOR,VERSION_RELEASE,VERSION_DEV,VERSION_TIME,DESCRIPTION)
VALUES ('458202233263219661970198300000000000',1,0,0,15,0,'Initialization from install script');

-------------------Create tables for business space
	
	CREATE TABLE @SCHEMA@.BSP_USER_DATA_T (
		  USER_DN                 VARCHAR(1024) NOT NULL ,
		  EXTENSION               BLOB(10K) ,
		  PRIMARY KEY ( USER_DN )
		) ;

